
/**************************************************************************************************************************************************
*       Copyright  2009-2018 Texas Instruments Incorporated - http://www.ti.com/                                                                 *
***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*                                                                                                                                                 *
**************************************************************************************************************************************************/

/**************************************************************************************************************************************************
*                                 Included Headers                                                                                                *
**************************************************************************************************************************************************/
#include <msp430.h>
#include <eusci_b_i2c.h>
#include <I2C_functions.h>
/* TI includes. */
#include "driverlib.h"
//#include "FreeRTOS.h"
//#include "task.h"
/**************************************************************************************************************************************************
*                                 Definitions                                                                                                     *
**************************************************************************************************************************************************/

/**************************************************************************************************************************************************
*                                 Prototypes                                                                                                      *
**************************************************************************************************************************************************/


void I2C_init (void)
{


    EUSCI_B_I2C_initMasterParam param;
    param.selectClockSource=EUSCI_B_I2C_CLOCKSOURCE_SMCLK;
    param.i2cClk=CS_getSMCLK();
    param.dataRate=EUSCI_B_I2C_SET_DATA_RATE_400KBPS;//param.dataRate=EUSCI_B_I2C_SET_DATA_RATE_400KBPS;

    param.byteCounterThreshold=0x0;
    param.autoSTOPGeneration=EUSCI_B_I2C_NO_AUTO_STOP;

    EUSCI_B_I2C_initMaster(EUSCI_B0_BASE,&param);

    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,0x7F);  //Broadcast I2C slave address for TPS2388
    EUSCI_B_I2C_setMode(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_MODE);
    EUSCI_B_I2C_enable(EUSCI_B0_BASE);
    EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);

    //Enable master Receive interrupt
    EUSCI_B_I2C_enableInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0 +EUSCI_B_I2C_NAK_INTERRUPT);
}

#if(1)

uint8_t tps_WriteI2CReg (uint8_t i2cAddress,uint8_t registerAddress, uint8_t Value)
{

    uint8_t rtn;
    uint8_t value[2];

    value[0] = registerAddress;
    value[1] = Value;

    rtn = Write_I2C (i2cAddress, value, 2);

    return (rtn);
}


uint8_t tps_ReadI2CReg (uint8_t i2cAddress,uint8_t registerAddress, uint8_t *readValue)
{
    uint8_t rtn;

    rtn = Read_I2C(i2cAddress, registerAddress, readValue, 1);

    return (rtn);
}


uint8_t tps_ReadI2CMultiple (uint8_t i2cAddress, uint8_t registerAddress, uint8_t *readValue, uint8_t numReadBytes)
{
    uint8_t rtn;


    rtn = Write_Read_I2C(i2cAddress, &registerAddress, 1, readValue, numReadBytes);

    return (rtn);

}


#endif




#if(0)
/**************************************************************************************************************************************************
*                                 I2C Write Routines                                                                                              *
**************************************************************************************************************************************************/
uint8_t tps_WriteI2CReg (uint8_t i2cAddress, uint8_t registerAddress, uint8_t Value)
{
uint8_t rtn = 1;

//Chandra's
#if(1)
uint8_t value[2];

    value[0] = registerAddress;
    value[1] = Value;

    while(EUSCI_B_I2C_isBusBusy(EUSCI_B0_BASE));

    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);
    EUSCI_B_I2C_masterSendMultiByte(EUSCI_B0_BASE,value,sizeof(value));


#endif


//Penny's
#if(0)
    //Specify slave address
    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);

    /* Enable and clear the interrupt flag */
   //EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_RECEIVE_INTERRUPT0);

    //Set in transmit mode
    EUSCI_B_I2C_setMode(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_MODE);

    /* Clear any existing interrupt flag PL */
    EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0);

    /* Initiate start and send first character */
    EUSCI_B_I2C_masterSendMultiByteStart(EUSCI_B0_BASE,registerAddress);


    EUSCI_B_I2C_masterSendMultiByteFinish(EUSCI_B0_BASE,Value);
#endif

   return (rtn);
}

uint8_t tps_WriteI2CMultiple (uint8_t i2cAddress, uint8_t registerAddress, uint8_t *writeValues, uint8_t numWriteBytes)
{
	uint8_t rtn = 0;
	uint8_t value[32];

	value[0]=registerAddress;
	memcpy(&value[1],writeValues,numWriteBytes);
    while(EUSCI_B_I2C_isBusBusy(EUSCI_B0_BASE));
    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);
    rtn = EUSCI_B_I2C_masterSendMultiByte(EUSCI_B0_BASE,value,(numWriteBytes+1));
    return (rtn);
}

/**************************************************************************************************************************************************
*                                 I2C Read Routines                                                                                               *
**************************************************************************************************************************************************/
uint8_t tps_ReadI2CReg (uint8_t i2cAddress, uint8_t registerAddress, uint8_t *readValue)
{

    //Chandra's
#if(0)
    uint8_t rtn=0;

    uint8_t data[2];

    uint8_t timeout =0xFF;
    //taskENTER_CRITICAL();
    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);
    rtn=EUSCI_B_I2C_masterSendSingleByteWithTimeout(EUSCI_B0_BASE,registerAddress,1000); //Read General Mask Reg
    if (rtn && 0x02)
    {
    EUSCI_B_I2C_masterReceiveStart(EUSCI_B0_BASE);
    //Poll for receive interrupt flag.
    do
    {
        rtn=HWREG16(EUSCI_B0_BASE + OFS_UCBxIFG);
    }while(!(rtn & UCRXIFG) && --timeout);
    if (timeout !=0x0)
    {

        //Send stop condition.
        HWREG16(EUSCI_B0_BASE + OFS_UCBxCTLW0) |= UCTXSTP;
        while(!(HWREG16(EUSCI_B0_BASE + OFS_UCBxCTLW0) & UCTXSTP));

        *readValue = HWREG16(EUSCI_B0_BASE + OFS_UCBxRXBUF);

        //data[0]=EUSCI_B_I2C_masterReceiveMultiByteNext(EUSCI_B0_BASE);
//        data[1]=EUSCI_B_I2C_masterReceiveMultiByteFinish(EUSCI_B0_BASE);


//       *readValue = EUSCI_B_I2C_masterReceiveSingle(EUSCI_B0_BASE);
        //Penny
        //memcpy(readValue,data,sizeof(data));
    }
    }
    //taskEXIT_CRITICAL();
    return (rtn && 0xFF);
#endif
    //Penny
#if(1)
    uint8_t rtn=0;

    //Specify slave address
    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);

    /* Enable and clear the interrupt flag */
    //EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_RECEIVE_INTERRUPT0);

    //Set in transmit mode
    EUSCI_B_I2C_setMode(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_MODE);

    /* Clear any existing interrupt flag PL */
    EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE,EUSCI_B_I2C_TRANSMIT_INTERRUPT0);

    EUSCI_B_I2C_masterSendSingleByte(EUSCI_B0_BASE,registerAddress);

    *readValue = EUSCI_B_I2C_masterReceiveSingleByte(EUSCI_B0_BASE);



return rtn;
#endif

}

uint8_t tps_ReadI2CMultiple (uint8_t i2cAddress, uint8_t registerAddress, uint8_t *readValue, uint8_t numReadBytes)
{
	 	uint8_t i,rtn=0;
	    uint8_t data[32];
	    uint8_t timeout =0xFF;

	    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE,i2cAddress);
	    EUSCI_B_I2C_masterSendSingleByteWithTimeout(EUSCI_B0_BASE,registerAddress,1000); //Read General Mask Reg

		EUSCI_B_I2C_masterReceiveStart(EUSCI_B0_BASE);
		//Poll for receive interrupt flag.
		do
		{
			rtn=HWREG16(EUSCI_B0_BASE + OFS_UCBxIFG);
		}while(!(rtn & UCRXIFG) && --timeout);
		if (timeout !=0x0)
		{
			timeout =0xFF;
			for (i=0;i<numReadBytes-1;i++)
			{
				data[i]=EUSCI_B_I2C_masterReceiveMultiByteNext(EUSCI_B0_BASE);
				//Poll for receive interrupt flag.

			}
				data[numReadBytes-1]=EUSCI_B_I2C_masterReceiveMultiByteFinish(EUSCI_B0_BASE);
				memcpy(readValue,data,numReadBytes);
		}

	    return (rtn);

}






/*
#pragma vector = USCIAB0TX_VECTOR       //I2C - UCB0RXIFG and UCB0TXIFG interrupts; UART/SPI - TX Interrupt
__interrupt void USCI0TX_ISR(void)
{
    uint8_t wakeCpu = DONT_WAKE_CPU;

    // Is this an I2C Interrupt???
    if (((IE2 & UCB0RXIE) && (IFG2 & UCB0RXIFG)) ||
        ((IE2 & UCB0TXIE) && (IFG2 & UCB0TXIFG)))
    {
        wakeCpu = i2c_ISR();
    }

    if (wakeCpu)
    {
        __bic_SR_register_on_exit(CPUOFF + GIE);      // Exit LPM0
    }

}

#pragma vector = USCIAB0RX_VECTOR       //I2C - Status interrupts; SPI - RX Interrupt
__interrupt void USCI0RX_ISR(void)
{
    uint8_t wakeCpu = DONT_WAKE_CPU;

    // Is this an I2C NACK Interrupt???
    if ((UCB0I2CIE & UCNACKIE) && (UCB0STAT & UCNACKIFG))
    {
        wakeCpu = i2c_stat_isr ();
    }

    if (wakeCpu)
    {
        __bic_SR_register_on_exit(CPUOFF + GIE);      // Exit LPM0
    }

}
*/
#endif
