
/**************************************************************************************************************************************************
*       Copyright  2009-2018 Texas Instruments Incorporated - http://www.ti.com/                                                                 *
***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*                                                                                                                                                 *
**************************************************************************************************************************************************/

/**************************************************************************************************************************************************
*                                 Included Headers                                                                                                *
**************************************************************************************************************************************************/

#include "usci_uart.h"

/**************************************************************************************************************************************************
*                                 Definitions                                                                                                     *
**************************************************************************************************************************************************/

uint8_t RXFLG = 0;
uint8_t  uartRxData[2] = {0,0};
uint8_t  rxDataIdx = 0;
uint8_t readValue;
/**************************************************************************************************************************************************
*                                 Prototypes                                                                                                      *
**************************************************************************************************************************************************/



void uart_A0_init(void)
{

    EUSCI_A_UART_initParam param = {0};
    param.selectClockSource = EUSCI_A_UART_CLOCKSOURCE_ACLK;
    param.clockPrescalar = 3;
    param.firstModReg = 0;
    param.secondModReg = 146;
    param.parity = EUSCI_A_UART_NO_PARITY;
    param.msborLsbFirst = EUSCI_A_UART_LSB_FIRST;
    param.numberofStopBits = EUSCI_A_UART_ONE_STOP_BIT;
    param.uartMode = EUSCI_A_UART_MODE;
    param.overSampling = EUSCI_A_UART_LOW_FREQUENCY_BAUDRATE_GENERATION;

    if(STATUS_FAIL == EUSCI_A_UART_init(EUSCI_A0_BASE, &param))
    {
        return;
    }

    EUSCI_A_UART_enable(EUSCI_A0_BASE);

    EUSCI_A_UART_clearInterrupt(EUSCI_A0_BASE,
                                EUSCI_A_UART_RECEIVE_INTERRUPT);

    // Enable USCI_A0 RX interrupt
    EUSCI_A_UART_enableInterrupt(EUSCI_A0_BASE,
                                 EUSCI_A_UART_RECEIVE_INTERRUPT); // Enable interrupt


}


void uart_A1_init(void)
{

    GPIO_setAsPeripheralModuleFunctionInputPin(
        GPIO_PORT_P2,
        GPIO_PIN5 + GPIO_PIN6,
        GPIO_SECONDARY_MODULE_FUNCTION);

    EUSCI_A_UART_initParam param = {0};
    param.selectClockSource = EUSCI_A_UART_CLOCKSOURCE_ACLK;
    param.clockPrescalar = 3;
    param.firstModReg = 0;
    param.secondModReg = 92;
    param.parity = EUSCI_A_UART_NO_PARITY;
    param.msborLsbFirst = EUSCI_A_UART_LSB_FIRST;
    param.numberofStopBits = EUSCI_A_UART_ONE_STOP_BIT;
    param.uartMode = EUSCI_A_UART_MODE;
    param.overSampling = EUSCI_A_UART_LOW_FREQUENCY_BAUDRATE_GENERATION;

    if(STATUS_FAIL == EUSCI_A_UART_init(EUSCI_A0_BASE, &param))
    {
        return;
    }

    EUSCI_A_UART_enable(EUSCI_A1_BASE);

    EUSCI_A_UART_clearInterrupt(EUSCI_A1_BASE,
                                EUSCI_A_UART_RECEIVE_INTERRUPT);

    // Enable USCI_A1 RX interrupt
    EUSCI_A_UART_enableInterrupt(EUSCI_A1_BASE,
                                 EUSCI_A_UART_RECEIVE_INTERRUPT); // Enable interrupt

}

/*************************************************************************************************************************************************
*  uart_putc
**************************************************************************************************************************************************/
/*!
* @brief This function will place one ASCII character on the UART TX line
*
* The function sends a single character onto the UART TX line. It will wait until the UART status indicates the TX line is empty before sending
* the character.
*
* @note This function returns when the character is placed into the UART TXBUF. It is actually sent on the line by the hardware sometime after
* this occurs. Therefore, a return from this function does NOT indicate the character has been sent on the hardware line.
*
* @param[out]   c             Character to send onto the UART TX line
*
* @return  None
**************************************************************************************************************************************************/
void uart_putc(unsigned char c)
{
    EUSCI_A_UART_transmitData(EUSCI_A0_BASE, c);
}

/*************************************************************************************************************************************************
*  uart_puts
**************************************************************************************************************************************************/
/*!
* @brief This function will place an ASCII string (ends with a NULL character) on the UART TX line
*
* The function sends an ASCII string onto the UART TX line. It will call the uart_putc() function for each byte until a NULL characetre (\0) is
* found in the string indicating end of string. NOte that this function does not place the system in low power mode between bytes. It will end up
* trying to send each byte and waiting until the previous byte is sent. Therefore, this is not the most power efficient algorithm ever developed
* for UART transmissions.
*
* @note This function returns when the last character is placed into the UART TXBUF. It may not have actually been sent on the line by the hardware
* when the function returns. Therefore, a return from this function does NOT indicate the character has been sent on the hardware line.
*
* @param[out]   str             String to send onto the UART TX line
*
* @return  None
**************************************************************************************************************************************************/

void uart_puts(char *str)
{
    while(*str)
    uart_putc(*str++);      //Advance though string till end
}

/*************************************************************************************************************************************************
*  uart_putLong
**************************************************************************************************************************************************/
/*!
* @brief This function will convert an unsigned long integer into ASCII characters and place them on the UART TX line.
*
* The function will convert a number into ASCII characters and place them on the UART TX line. This allows a user to "print" a numerical value.
* As this handles unsigned long integers, it does not deal with fractions or negative numbers.
*
* @param[out]   number             Unsigned long number to convert into ASCII characters and place on UART TX Line
*
* @return  None
**************************************************************************************************************************************************/

void uart_putLong (unsigned long number)
{
unsigned char start = 0;
unsigned long target = 1000000000;
unsigned char value;

    while (target > 1)
    {
        if (number >= target)
        {
            start = 1;
            value = number / target;
            uart_putc ('0' + value);
            number = number - (target * value);
        }
        else
        {
            if (start)
                uart_putc ('0');
        }
        target = target / 10;
    }

    uart_putc('0' + number);
}


void uartPutHex (unsigned char hexByte)
{
unsigned char target;

    target = hexByte >> 4;
    if (target < 10)
        uart_putc('0' + target);
    else
        uart_putc('A' + (target - 10));

    target = hexByte & 0xf;
    if (target < 10)
        uart_putc('0' + target);
    else
        uart_putc('A' + (target - 10));

}




#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
    switch(__even_in_range(UCA0IV,USCI_UART_UCTXCPTIFG))
    {
        case USCI_NONE: break;

        case USCI_UART_UCRXIFG:

            uartRxData[rxDataIdx] = HWREG16(EUSCI_A0_BASE + OFS_UCAxRXBUF);
            rxDataIdx++;
            if(rxDataIdx == 2)
            {
                rxDataIdx = 0;
                RXFLG =1;
//                tps_ReadI2CReg(uartRxData[0],uartRxData[1],&readValue);
//                uartPutHex (readValue);

            }



//            // Read RX buffer
//            pRxData[rxDataIdx] = EUSCI_A_UART_receiveData(EUSCI_A0_BASE);
//            rxDataIdx++;
//
//            if(rxDataIdx == rxDataLen)
//            {
//                if(uartCb != 0)
//                {
//                    uartCb(pRxData, rxDataLen);
//                }
//
//                rxDataIdx = 0;
//            }

        break;

        case USCI_UART_UCTXIFG: break;
        case USCI_UART_UCSTTIFG: break;
        case USCI_UART_UCTXCPTIFG: break;
    }
}

