/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <tps238x.h>
#include "I2C_functions.h"
#include "driverlib.h"
#include "I2C_functions.h"
#include "usci_uart.h"
#include "sys_init.h"


//******************************************************************************
//!
//!   Empty Project that includes driverlib
//!
//******************************************************************************

uint16_t i;
uint8_t RXData = 0, TXData = 0;
uint8_t check = 1;
uint8_t Add = 0;
uint8_t RTN = 0xFF;
uint8_t OPM;
uint8_t DCDT;
TPS238X_Interrupt_Register_t intStatus;
TPS238x_Ports_t powerEnablePortEvents, powerGoodPortEvents, detectionPortEvents, classificationPortEvents, pcutPortEvents, disconnectPortEvents, inrushPortEvents, ilimPortEvents;
TPS238x_Detection_Status_t detectStatus;
TPS238x_Classification_Status_t classStatus;
uint8_t supplyEvents;
uint8_t  devNum;
uint8_t sysPortNum,sysPortNum1, sysPortNum2, sysPortNum3, sysPortNum4,sysPortNum5, sysPortNum6, sysPortNum7, sysPortNum8;
uint16_t voltage1,voltage2,current1,current2;
uint8_t CCResult;
uint8_t RAMcontrol;
uint8_t assignedclass;
uint8_t powerallocation;
uint8_t foldback;
uint8_t detectClassE;
uint8_t powerStatus;
uint8_t IntFlag;
#if (PRINT_STATUS == 1)
uint8_t detectR;
uint8_t connectcheck;
uint8_t PrintPower;
uint16_t voltage, current;
uint8_t  temperature;
unsigned long outNum;
#endif

void main(void)
{
    /* Stop Watchdog timer. */
    WDT_A_hold( __MSP430_BASEADDRESS_WDT_A__ );

    MSP430_init();


    __enable_interrupt();

    __delay_cycles(184000);   //Wait TPS23861 I2C to be operational

    //Mapping system port for each port
    sysPortNum1 = tps_RegisterPort (tps2388x_i2cAddList[0], TPS238X_PORT_1);
    sysPortNum2 = tps_RegisterPort (tps2388x_i2cAddList[0], TPS238X_PORT_2);
    sysPortNum3 = tps_RegisterPort (tps2388x_i2cAddList[0], TPS238X_PORT_3);
    sysPortNum4 = tps_RegisterPort (tps2388x_i2cAddList[0], TPS238X_PORT_4);

    sysPortNum5 = tps_RegisterPort (tps2388x_i2cAddList[1], TPS238X_PORT_1);
    sysPortNum6 = tps_RegisterPort (tps2388x_i2cAddList[1], TPS238X_PORT_2);
    sysPortNum7 = tps_RegisterPort (tps2388x_i2cAddList[1], TPS238X_PORT_3);
    sysPortNum8 = tps_RegisterPort (tps2388x_i2cAddList[1], TPS238X_PORT_4);


    // Get the interrupt and event statuses. Clear the event statuses while reading
    tps_GetDeviceInterruptStatus (tps2388x_i2cAddList[0], &intStatus);
    tps_GetDeviceInterruptStatus (tps2388x_i2cAddList[1], &intStatus);

    tps_GetDeviceAllInterruptEvents (tps2388x_i2cAddList[0], TPS_ON, &powerEnablePortEvents, &powerGoodPortEvents, &detectionPortEvents,
                                            &classificationPortEvents, &pcutPortEvents, &disconnectPortEvents,
                                            &inrushPortEvents, &ilimPortEvents, &supplyEvents);
    tps_GetDeviceAllInterruptEvents (tps2388x_i2cAddList[1], TPS_ON, &powerEnablePortEvents, &powerGoodPortEvents, &detectionPortEvents,
                                                &classificationPortEvents, &pcutPortEvents, &disconnectPortEvents,
                                                &inrushPortEvents, &ilimPortEvents, &supplyEvents);


    PSE_TPS238X_init(OPERATING_MODE_SEMI_AUTO);

    uart_puts ("Welcome to the POE 23880 - Semi-Auto Mode for Basic Applications\r\n\n\n");



    while(1)
    {
        if (!IntFlag)
        {
            __enable_interrupt();
            LPM0;
            __no_operation();
        }
        else
        {
            IntFlag = 0;

            for (devNum = 0; devNum < (NUM_OF_TPS2388x*NUM_OF_QUARD); devNum++)
            {
                // Get the interrupt and event statuses. Clear the event statuses while reading
                tps_GetDeviceInterruptStatus (tps2388x_i2cAddList[devNum], &intStatus);


                tps_GetDeviceAllInterruptEvents (tps2388x_i2cAddList[devNum], TPS_ON, &powerEnablePortEvents, &powerGoodPortEvents, &detectionPortEvents,
                                                        &classificationPortEvents, &pcutPortEvents, &disconnectPortEvents,
                                                        &inrushPortEvents, &ilimPortEvents, &supplyEvents);

                if(intStatus.CLASC_Classification_Cycle)
                {
                    uint8_t i;
                    uint8_t target;


                    target = classificationPortEvents;


                    for (i = TPS238X_PORT_1; i <= TPS238X_PORT_4; i++)
                    {
                        if (target & 0x1)
                        {
                            sysPortNum = tps_GetSystemPortNumber (tps2388x_i2cAddList[devNum], (TPS238x_PortNum_t)i);

                            tps_GetPortDetectRequestedClassStatus(sysPortNum, &detectStatus, &classStatus);

                            if ((classStatus != CLASS_OVERCURRENT) &&
                                (classStatus != CLASS_UNKNOWN) &&
                                (classStatus != CLASS_MISMATCH))
                            {

                                if((tps_GetPortPowerEnableStatus(sysPortNum) == TPS_OFF))
                                {
                                    //Turn on 2 pair port
                                    if(sysPortNum < 4)
                                    {
                                        tps_SetPortPower(sysPortNum, TPS_ON);
                                        __delay_cycles(10000);
                                    }
                                    else
                                    {
                                        //Turn on 4 pair port
                                        if((sysPortNum == 4) | (sysPortNum == 5))
                                        {
                                            tps_SetDevicePowerOn(tps2388x_i2cAddList[devNum], 0x03);
                                        }

                                        if((sysPortNum == 6) | (sysPortNum == 7))
                                        {
                                            tps_SetDevicePowerOn(tps2388x_i2cAddList[devNum], 0x0C);
                                        }

                                        __delay_cycles(10000);
                                    }

                                }

                            }
                        }

                        target = target >> 1;
                    }
                }

                if(intStatus.DISF_Disconnect_Event)
                {
                    uint8_t i;
                    uint8_t target;

                    target = disconnectPortEvents;

                    for (i = TPS238X_PORT_1; i <= TPS238X_PORT_4; i++)
                    {
                      //insert code here if wanting to report DC disconnect

                    }
                }


                if(intStatus.IFAULT_PCUT_ILIM_Fault)
                {
                    uint8_t i;
                    uint8_t targetPcut;
                    uint8_t targetILIM;
                    uint8_t target4Pcut;

                    targetPcut = pcutPortEvents;
                    targetILIM = ilimPortEvents;
                    target4Pcut = (supplyEvents & 0x0f)>>2;


                    if(target4Pcut & 0x01)
                    {
                        uart_puts("4P PCUT Fault Channel");
                        uart_putLong ((unsigned long)( (4 * devNum) + 1));
                        uart_putLong ((unsigned long)( (4 * devNum) + 2));
                        uart_puts ("\r\n");
                    }

                    if(target4Pcut & 0x10)
                    {
                        uart_puts("4P PCUT Fault Channel");
                        uart_putLong ((unsigned long)( (4 * devNum) + 3));
                        uart_putLong ((unsigned long)( (4 * devNum) + 4));
                        uart_puts ("\r\n");

                    }

                    for(i = TPS238X_PORT_1; i <= TPS238X_PORT_4; i++)
                    {
                        if (targetPcut & 0x01)
                        {
                            uart_puts("PCUT Fault Port");
                            uart_putLong ((unsigned long)(i + (4 * devNum) - 1));
                            uart_puts ("\r\n");
                        }

                        if(targetILIM & 0x01)
                        {
                            uart_puts("ILIM Fault Port");
                            uart_putLong ((unsigned long)(i + (4 * devNum) - 1));
                            uart_puts ("\r\n");
                        }

                        targetPcut >>= 1;
                        targetILIM >>= 1;

                    }

                }


                if(intStatus.INRF_Inrush_Fault)
                {
                    uint8_t i;
                    uint8_t target;

                    target = inrushPortEvents;
                    for (i = TPS238X_PORT_1; i <= TPS238X_PORT_4; i++)
                    {
                        if (target & 0x1)
                        {
                            uart_puts ("Start Fault Port");
                            uart_putLong ((unsigned long)(i + (4 * devNum) - 1));
                            uart_puts ("\r\n");
                        }
                        target >>= 1;
                    }
                }


                if(intStatus.SUPF_Supply_Event_Fault)
                {
                    uint8_t target;
                    target = supplyEvents;

                    if(target & 0x01)
                    {
                        uart_puts("SRAM memory fault occurred");
                        uart_puts("\r\n");

                    }

                    else if(target & 0x10)
                    {
                        uart_puts("VPWR undervlotage occurred");
                        uart_puts("\r\n");
                    }
                    else if(target & 0x20)
                    {
                        uart_puts("VDD undervlotage warning occurred");
                        uart_puts("\r\n");
                    }

                    else if(target & 0x40)
                    {
                        uart_puts("VDD undervlotage  occurred");
                        uart_puts("\r\n");
                    }

                    else if(target & 0x80)
                    {
                        uart_puts("Thermal shutdown occurred");
                        uart_puts("\r\n");
                    }

                }
            }
        }  //if !(IntFlag)




#if(PRINT_STATUS)
                if(PrintPower)
                {
                    PrintPower = 0;
                    tps_GetDeviceInputVoltage (tps2388x_i2cAddList[0], &voltage);
                    uart_puts ("Input Voltage: ");
                    outNum = ((unsigned long)voltage * 3662) / 1000;
                    uart_putLong (outNum);
                    uart_puts ("mV \r\n");

                    tps_GetDeviceTemperature (tps2388x_i2cAddList[0], &temperature);
                    uart_puts ("Device Temperature: ");
                    outNum = CONVERT_TEMP((unsigned long)temperature);
                    uart_putLong (outNum);
                    uart_puts (" degrees C\r\n\n");

                    tps_GetDeviceFirmwareRev(tps2388x_i2cAddList[0],&firmwareversion);
                    uart_puts ("Firmware Revision: ");
                    uartPutHex(firmwareversion);
                    uart_puts ("\r\n\n");

                    for(sysPortNum = sysPortNum1; sysPortNum <= sysPortNum8; sysPortNum++)
                    {

                        if (tps_GetPortPowerGoodStatus(sysPortNum) == TPS_ON)
                        {
                            uart_puts ("Channel ");
                            uartPutHex(sysPortNum + 1);
                            uart_puts (": ON \r\n");
                            tps_GetPortMeasurements (sysPortNum, &voltage, &current);
                            uart_puts ("   Voltage: ");
                            outNum = ((unsigned long)voltage * 3662) / 1000;
                            uart_putLong (outNum);
                            uart_puts ("mV      Current: ");
                            outNum = ((unsigned long)current * 70190) / 1000000;
                            uart_putLong (outNum);
                            uart_puts ("mA \r\n");
                        }
                        else
                        {
                            uart_puts ("Channel ");
                            uartPutHex(sysPortNum + 1);
                            uart_puts (": OFF \r\n");
                        }

                        tps_GetPortDetectionStatus(sysPortNum, &detectStatus);

                        switch(detectStatus)
                        {
                        case DETECT_UNKNOWN: uart_puts ("   Detection Status: UNKNOWN");
                        break;
                        case DETECT_SHORT_CIRCUIT: uart_puts ("   Detection Status: SHORT CIRCUIT");
                        break;
                        case DETECT_RESIST_LOW: uart_puts ("   Detection Status: RESISTANCE TOO LOW");
                        tps_GetPortDetectResistance(sysPortNum, &detectR);
                        outNum = ((unsigned long)detectR * 1953125) / 10000;
                        uart_puts ("   Detection Resistance: ");
                        uart_putLong (outNum);
                        uart_puts ("Ohm");
                        break;
                        case DETECT_RESIST_VALID: uart_puts ("   Detection Status: RESISTANCE VALID");
                        tps_GetPortDetectResistance(sysPortNum, &detectR);
                        outNum = ((unsigned long)detectR * 1953125) / 10000;
                        uart_puts ("   Detection Resistance: ");
                        uart_putLong (outNum);
                        uart_puts ("Ohm");
                        break;
                        case DETECT_RESIST_HIGH: uart_puts ("   Detection Status: RESISTANCE TOO HIGH");
                        tps_GetPortDetectResistance(sysPortNum, &detectR);
                        outNum = ((unsigned long)detectR * 1953125) / 10000;
                        uart_puts ("   Detection Resistance: ");
                        uart_putLong (outNum);
                        uart_puts ("Ohm");
                        break;
                        case DETECT_OPEN_CIRCUIT: uart_puts ("   Detection Status: OPEN CIRCUIT");
                        break;
                        case DETECT_MOSFET_FAULT: uart_puts ("   Detection Status: MOSFET FAULT");
                        break;
                        default:
                        break;
                        }

                        tps_GetPortRequestedClassificationStatus(sysPortNum,&classStatus);
                        switch(classStatus)
                        {
                        case 0x00: uart_puts ("\r\n   Classification Status: Unknown \r\n");
                        break;
                        case 0x01: uart_puts ("\r\n   Classification Status: Class 1 \r\n");
                        break;
                        case 0x02: uart_puts ("\r\n   Classification Status: Class 2 \r\n");
                        break;
                        case 0x03: uart_puts ("\r\n   Classification Status: Class 3 \r\n");
                        break;
                        case 0x04: uart_puts ("\r\n   Classification Status: Class 4 \r\n");
                        break;
                        case 0x05: uart_puts ("\r\n   Classification Status: Class 0 \r\n");
                        break;
                        case 0x06: uart_puts ("\r\n   Classification Status: Class 0 \r\n");
                        break;
                        case 0x07: uart_puts ("\r\n   Classification Status: Over current \r\n");
                        break;
                        case 0x08: uart_puts ("\r\n   Classification Status: Class 5, 4 Pair Single Signature \r\n");
                        break;
                        case 0x09: uart_puts ("\r\n   Classification Status: Class 6, 4 Pair Single Signature \r\n");
                        break;
                        case 0x0A: uart_puts ("\r\n   Classification Status: Class 7, 4 Pair Single Signature \r\n");
                        break;
                        case 0x0B: uart_puts ("\r\n   Classification Status: Class 8, 4 Pair Single Signature \r\n");
                        break;
                        case 0x0C: uart_puts ("\r\n   Classification Status: Class 4+, Type 1 Limited \r\n");
                        break;
                        case 0x0D: uart_puts ("\r\n   Classification Status: Class 5, 4 Pair Dual Signature \r\n");
                        break;
                        case 0x0E: uart_puts ("\r\n   Classification Status: UNKNOWN \r\n");
                        break;
                        case 0x0F: uart_puts ("\r\n   Classification Status: Class Mismatch \r\n");
                        break;
                        default:
                        break;
                        }


                        if(sysPortNum > sysPortNum4)
                        {
                            tps_GetPortConnectionCheckStatus(sysPortNum,&connectcheck);

                            switch(connectcheck)
                            {
                            case 0x00: uart_puts("   Connection Check Status: Unknown \r\n");
                            break;
                            case 0x01: uart_puts("   Connection Check Status: 4-Pair Single Signature PD \r\n");
                            break;
                            case 0x02: uart_puts("   Connection Check Status: 4-Pair Dual Signature PD \r\n");
                            break;
                            case 0x03: uart_puts("   Connection Check Status: Invalid Signature Detection \r\n");
                            break;
                            default:
                            break;
                            }
                        }
                    }

#if (DETAILED_STATUS == 1)
            for (devNum = 0; devNum < (NUM_OF_TPS2388x*NUM_OF_QUARD); devNum++)
            {
                // read current value of all event registers (Do not clear)
                tps_GetDeviceAllInterruptEvents (tps2388x_i2cAddList[devNum], TPS_OFF, &powerEnablePortEvents, &powerGoodPortEvents, &detectionPortEvents,
                                                        &classificationPortEvents, &pcutPortEvents, &disconnectPortEvents,
                                                        &inrushPortEvents, &ilimPortEvents, &supplyEvents);

                uart_puts ("\n---- Event Registers -----Dev : ");
                uartPutHex (devNum);
                uart_puts ("------- \r\n0x");
                uartPutHex ((powerGoodPortEvents<<4) | powerEnablePortEvents);
                uart_puts ("   0x");
                uartPutHex ((classificationPortEvents << 4) | detectionPortEvents);
                uart_puts ("   0x");
                uartPutHex ((disconnectPortEvents << 4) | pcutPortEvents);
                uart_puts ("   0x");
                uartPutHex ((ilimPortEvents << 4) | inrushPortEvents);
                uart_puts ("   0x");
                uartPutHex ((*(unsigned char*)&supplyEvents << 4));
                uart_puts ("\r\n\n");

                uart_puts ("---- Port Status -----\r\n0x");
                sysPortNum = tps_GetSystemPortNumber (tps2388x_i2cAddList[devNum], TPS238X_PORT_1);
                tps_GetPortDetectRequestedClassStatus (sysPortNum, &detectStatus, &classStatus);
                uartPutHex ((classStatus<<4) | detectStatus);
                uart_puts ("   0x");
                sysPortNum = tps_GetSystemPortNumber (tps2388x_i2cAddList[devNum], TPS238X_PORT_2);
                tps_GetPortDetectRequestedClassStatus (sysPortNum, &detectStatus, &classStatus);
                uartPutHex ((classStatus<<4) | detectStatus);
                uart_puts ("   0x");
                sysPortNum = tps_GetSystemPortNumber (tps2388x_i2cAddList[devNum], TPS238X_PORT_3);
                tps_GetPortDetectRequestedClassStatus (sysPortNum, &detectStatus, &classStatus);
                uartPutHex ((classStatus<<4) | detectStatus);
                uart_puts ("   0x");
                sysPortNum = tps_GetSystemPortNumber (tps2388x_i2cAddList[devNum], TPS238X_PORT_4);
                tps_GetPortDetectRequestedClassStatus (sysPortNum, &detectStatus, &classStatus);
                uartPutHex ((classStatus<<4) | detectStatus);
                uart_puts ("\r\n\n");
                uart_puts ("---- Power Status -----\r\n0x");
                tps_GetDevicePowerStatus (tps2388x_i2cAddList[devNum], &powerEnablePortEvents, &powerGoodPortEvents);
                uartPutHex ((powerGoodPortEvents<<4) | powerEnablePortEvents);
                uart_puts ("\r\n\n");


                uart_puts ("------------------------------------------\r\n");
            }
#endif


                }
#endif

    } //end of while


}//end of main

uint8_t p1intcount = 0;
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)
{
    if (P1IFG & BIT3)
    {
        // Clear interrupt flags
        P1IFG = 0;

        IntFlag = 1;
//        p1intcount++;
    }

//    P1IFG &= ~(BIT3);
    __bic_SR_register_on_exit(LPM0_bits + GIE);
}




/**************************************************************************************************************************************************
* TIMER0_A1
**************************************************************************************************************************************************/
/*!
*
* @brief Timer Interrupt Routine
*
* @return None
**************************************************************************************************************************************************/
#pragma vector = TIMER0_A1_VECTOR
__interrupt void TIMER0_A1_ISR (void)
{
static unsigned long counter=0;

    TA0CTL &= ~TAIFG;

    if (++counter == 200)       // Every 10 seconds
    {
        counter = 0;
#if (PRINT_STATUS == 1)
        PrintPower = 1;
#endif
    }


    // Double check interrupt - Is P2.3 Low and P2.3 IFG not present?
    if ((!(P1IFG & BIT3)) && (!(P1IN & BIT3)))
    {
        IntFlag = 1;
    }

    LPM0_EXIT;
}
